package com.uas.platform.core.data.repository;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;

import org.springframework.data.jpa.repository.support.JpaEntityInformation;
import org.springframework.data.jpa.repository.support.JpaEntityInformationSupport;
import org.springframework.data.jpa.repository.support.SimpleJpaRepository;
import org.springframework.data.repository.NoRepositoryBean;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.uas.platform.core.data.support.OperationLogger;
import com.uas.platform.core.model.Operation;

/**
 * 
 * @see GenericRepository
 * @see LogRepository
 * @see SimpleJpaRepository
 * 
 * @author yingp
 * 
 * @param <T>
 * @param <ID>
 */
@NoRepositoryBean
public class SimpleGenericRepository<T, ID extends Serializable> extends SimpleJpaRepository<T, ID> implements
		GenericRepository<T, ID> {

	public SimpleGenericRepository(Class<T> domainClass, EntityManager em) {
		super(domainClass, em);
		this.em = em;
		this.logger = OperationLogger.getLogger(domainClass);
		this.entityInformation = JpaEntityInformationSupport.getEntityInformation(domainClass, em);
	}

	private final EntityManager em;
	private final JpaEntityInformation<T, ?> entityInformation;
	private final OperationLogger logger;

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#saveAndLog(java.lang
	 * .Object)
	 */
	@Transactional
	public <S extends T> S saveAndLog(S entity) {
		if (this.entityInformation.isNew(entity)) {
			this.logger.status(entity, Operation.INSERT);
			em.persist(entity);
			em.persist(logger.insert(entity));
			return entity;
		}
		return updateAndLog(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#saveAndLog(java.lang
	 * .Iterable)
	 */
	@Transactional
	public <S extends T> List<S> saveAndLog(Iterable<S> entities) {
		List<S> result = new ArrayList<S>();
		if (entities != null) {
			for (S entity : entities) {
				result.add(saveAndLog(entity));
			}
			flush();
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#updateAndLog(java.lang
	 * .Object)
	 */
	@Transactional
	public <S extends T> S updateAndLog(S entity) {
		Assert.notNull(entity);
		entity = em.merge(entity);
		em.persist(logger.update(entity));
		return entity;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#updateAndLog(java.lang
	 * .Iterable)
	 */
	@Transactional
	public <S extends T> List<S> updateAndLog(Iterable<S> entities) {
		List<S> result = new ArrayList<S>();
		if (entities != null) {
			for (S entity : entities) {
				result.add(updateAndLog(entity));
			}
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#submitAndLog(java.lang
	 * .Object)
	 */
	@Transactional
	public <S extends T> S submitAndLog(S entity) {
		Assert.notNull(entity);
		em.persist(logger.submit(entity));
		return em.merge(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#submitAndLog(java.io
	 * .Serializable)
	 */
	@Transactional
	public T submitAndLog(ID id) {
		Assert.notNull(id, "The given id must not be null!");
		T entity = findOne(id);
		if (entity != null) {
			return submitAndLog(entity);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#submitAndLog(java.lang
	 * .Iterable)
	 */
	@Transactional
	public <S extends T> List<S> submitAndLog(Iterable<S> entities) {
		List<S> result = new ArrayList<S>();
		if (entities != null) {
			for (S entity : entities) {
				result.add(submitAndLog(entity));
			}
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#auditAndLog(java.lang
	 * .Object)
	 */
	@Transactional
	public <S extends T> S auditAndLog(S entity) {
		Assert.notNull(entity);
		em.persist(logger.audit(entity));
		return em.merge(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#auditAndLog(java.io.
	 * Serializable)
	 */
	@Transactional
	public T auditAndLog(ID id) {
		Assert.notNull(id, "The given id must not be null!");
		T entity = findOne(id);
		if (entity != null) {
			return auditAndLog(entity);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#auditAndLog(java.lang
	 * .Iterable)
	 */
	@Transactional
	public <S extends T> List<S> auditAndLog(Iterable<S> entities) {
		List<S> result = new ArrayList<S>();
		if (entities != null) {
			for (S entity : entities) {
				result.add(auditAndLog(entity));
			}
		}
		return result;
	}

	@Transactional
	public <S extends T> S unAuditAndLog(S entity) {
		Assert.notNull(entity);
		em.persist(logger.unAudit(entity));
		return em.merge(entity);
	}

	@Transactional
	public T unAuditAndLog(ID id) {
		Assert.notNull(id, "The given id must not be null!");
		T entity = findOne(id);
		if (entity != null) {
			return unAuditAndLog(entity);
		}
		return null;
	}

	@Transactional
	public <S extends T> List<S> unAuditAndLog(Iterable<S> entities) {
		List<S> result = new ArrayList<S>();
		if (entities != null) {
			for (S entity : entities) {
				result.add(unAuditAndLog(entity));
			}
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#replyAndLog(java.lang
	 * .Object)
	 */
	@Transactional
	public <S extends T> S replyAndLog(S entity) {
		Assert.notNull(entity);
		em.persist(logger.reply(entity));
		return em.merge(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#replyAndLog(java.io.
	 * Serializable)
	 */
	@Transactional
	public T replyAndLog(ID id) {
		Assert.notNull(id, "The given id must not be null!");
		T entity = findOne(id);
		if (entity != null) {
			return replyAndLog(entity);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#replyAndLog(java.lang
	 * .Iterable)
	 */
	@Transactional
	public <S extends T> List<S> replyAndLog(Iterable<S> entities) {
		List<S> result = new ArrayList<S>();
		if (entities != null) {
			for (S entity : entities) {
				result.add(replyAndLog(entity));
			}
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#deleteAndLog(java.lang
	 * .Object)
	 */
	@Transactional
	public void deleteAndLog(T entity) {
		Assert.notNull(entity, "The entity must not be null!");
		em.persist(logger.delete(entity));
		em.remove(em.contains(entity) ? entity : em.merge(entity));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#deleteAndLog(java.lang
	 * .Iterable)
	 */
	@Transactional
	public void deleteAndLog(Iterable<? extends T> entities) {
		if (entities != null) {
			for (T entity : entities) {
				deleteAndLog(entity);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.uas.scm.common.data.repository.LogRepository#deleteAndLog(java.io
	 * .Serializable)
	 */
	@Transactional
	public void deleteAndLog(ID id) {
		Assert.notNull(id, "The given id must not be null!");
		T entity = findOne(id);
		if (entity != null)
			deleteAndLog(entity);
	}

}
