package com.uas.platform.core.data.support;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.springframework.util.Assert;

import com.uas.platform.core.exception.IllegalStatusException;
import com.uas.platform.core.model.Operation;
import com.uas.platform.core.model.OperationLog;
import com.uas.platform.core.model.Status;
import com.uas.platform.core.persistence.StatusColumn;

/**
 * 记录操作日志
 * 
 * @see Logger
 * @author yingp
 * 
 */
public class OperationLogger {

	public static OperationLogger getLogger(Class<?> cls) {
		return OperationLoggerManager.getLogger(cls);
	}

	private LoggerParams loggerParams;

	public OperationLogger(LoggerParams loggerParams) {
		this.loggerParams = loggerParams;
	}

	/**
	 * 记录新增
	 * 
	 * @see Operation
	 * @param object
	 * @return
	 */
	public Logable insert(Object object) {
		return info(object, Operation.INSERT);
	}

	public Logable insert(Object object, Long enUU, Long userUU) {
		return info(object, enUU, userUU, Operation.INSERT);
	}

	public Logable insert(Long key, Long enUU, Long userUU) {
		return info(key, enUU, userUU, Operation.INSERT);
	}

	/**
	 * 记录修改
	 * 
	 * @see Operation
	 * @param object
	 * @return
	 */
	public Logable update(Object object) {
		return info(object, Operation.UPDATE);
	}

	public Logable update(Object object, Long enUU, Long userUU) {
		return info(object, enUU, userUU, Operation.UPDATE);
	}

	public Logable update(Long key, Long enUU, Long userUU) {
		return info(key, enUU, userUU, Operation.UPDATE);
	}

	/**
	 * 记录刪除
	 * 
	 * @see Operation
	 * @param object
	 * @return
	 */
	public Logable delete(Object object) {
		return info(object, Operation.DELETE);
	}

	/**
	 * 从实体获取其状态
	 * 
	 * @see StatusColumn
	 * @param object
	 * @return
	 */
	private Status getStatus(Object object) {
		StatusColumn column = this.getLoggerParams().getStatusAnnotation();
		if (column != null) {
			Object invokeVal = invokeMethod(object, loggerParams.getStatusMethod());
			if (invokeVal != null) {
				return Status.valueOf(Integer.valueOf(invokeVal.toString()));
			}
		}
		return null;
	}

	/**
	 * 修改实体状态
	 * 
	 * @param object
	 *            实体
	 * @param operation
	 *            操作
	 * @return
	 */
	private void setStatus(Object object, Operation operation) {
		Status status = null;
		switch (operation) {
		case INSERT:
			status = Status.INPUTTING;
			break;
		case SUBMIT:
			status = Status.SUBMITTED;
			break;
		case AUDIT:
			status = Status.AUDITED;
			break;
		case UNAUDIT:
			status = Status.INPUTTING;
			break;
		case REPLY:
			status = Status.REPLIED;
			break;
		default:
			break;
		}
		if (status != null)
			invokeMethod(object, getLoggerParams().getStatusSetMethod(), status.value());
	}

	/**
	 * 按注解判断是否允许该操作
	 * 
	 * @see StatusColumn
	 * @param status
	 *            状态
	 * @param operation
	 *            操作
	 * @return
	 */
	private boolean allowOperation(Status status, Operation operation) {
		StatusColumn column = this.getLoggerParams().getStatusAnnotation();
		Status[] allowedStatus = new Status[] {};
		switch (operation) {
		case DELETE:
			allowedStatus = column.allowDelete();
			break;
		case AUDIT:
			allowedStatus = column.allowAudit();
			break;
		default:
			break;
		}
		if (allowedStatus.length == 0)
			return true;
		boolean allowed = false;
		for (Status allow : allowedStatus) {
			if (status == allow) {
				allowed = true;
				break;
			}
		}
		return allowed;
	}

	public Logable delete(Object object, Long enUU, Long userUU) {
		return info(object, enUU, userUU, Operation.DELETE);
	}

	public Logable delete(Long key, Long enUU, Long userUU) {
		return info(key, enUU, userUU, Operation.DELETE);
	}

	/**
	 * 记录提交
	 * 
	 * @see Operation
	 * @param object
	 * @return
	 */
	public Logable submit(Object object) {
		return info(object, Operation.SUBMIT);
	}

	public Logable submit(Object object, Long enUU, Long userUU) {
		return info(object, enUU, userUU, Operation.SUBMIT);
	}

	public Logable submit(Long key, Long enUU, Long userUU) {
		return info(key, enUU, userUU, Operation.SUBMIT);
	}

	/**
	 * 记录审核
	 * 
	 * @see Operation
	 * @param object
	 * @return
	 */
	public Logable audit(Object object) {
		return info(object, Operation.AUDIT);
	}

	public Logable audit(Object object, Long enUU, Long userUU) {
		return info(object, enUU, userUU, Operation.AUDIT);
	}

	public Logable audit(Long key, Long enUU, Long userUU) {
		return info(key, enUU, userUU, Operation.AUDIT);
	}
	
	/**
	 * 记录反审核
	 * 
	 * @see Operation
	 * @param object
	 * @return
	 */
	public Logable unAudit(Object object) {
		return info(object, Operation.UNAUDIT);
	}

	public Logable unAudit(Object object, Long enUU, Long userUU) {
		return info(object, enUU, userUU, Operation.UNAUDIT);
	}

	public Logable unAudit(Long key, Long enUU, Long userUU) {
		return info(key, enUU, userUU, Operation.UNAUDIT);
	}

	/**
	 * 记录回复
	 * 
	 * @see Operation
	 * @param object
	 * @return
	 */
	public Logable reply(Object object) {
		return info(object, Operation.REPLY);
	}

	public Logable reply(Object object, Long enUU, Long userUU) {
		return info(object, enUU, userUU, Operation.REPLY);
	}

	public Logable reply(Long key, Long enUU, Long userUU) {
		return info(key, enUU, userUU, Operation.REPLY);
	}

	/**
	 * 记录
	 * 
	 * @param object
	 *            实体对象
	 * @param operation
	 *            操作
	 * @throws IllegalStatusException
	 */
	public Logable info(Object object, Operation operation) throws IllegalStatusException {
		Assert.notNull(operation);
		if (operation != Operation.INSERT) {
			Status status = getStatus(object);
			if (status != null) {
				if (allowOperation(status, operation)) {
					setStatus(object, operation);
				} else
					throw new IllegalStatusException(object, status);
			}
		} else {
			if (this.getLoggerParams().getStatusAnnotation() != null)
				setStatus(object, operation);
		}
		return getLog(object, operation);
	}

	/**
	 * @param object
	 *            实体对象
	 * @param enUU
	 *            企业
	 * @param userUU
	 *            个人
	 * @param operation
	 *            操作
	 * @return
	 */
	public Logable info(Object object, Long enUU, Long userUU, Operation operation) {
		Assert.notNull(operation);
		return getLog(object, enUU, userUU, operation);
	}

	/**
	 * @param key
	 *            主键
	 * @param enUU
	 *            企业
	 * @param userUU
	 *            个人
	 * @param operation
	 *            操作
	 * @return
	 */
	public Logable info(Long key, Long enUU, Long userUU, Operation operation) {
		Assert.notNull(operation);
		return getLog(key, enUU, userUU, operation);
	}

	/**
	 * 
	 * @param object
	 *            实体对象
	 * @param operation
	 *            操作
	 * @param operationRemark
	 *            备注
	 */
	public Logable info(Object object, Operation operation, String operationRemark) {
		Assert.notNull(operation);
		return getLog(object, operation, operationRemark);
	}

	/**
	 * 
	 * @param object
	 *            实体对象
	 * @param operation
	 *            操作
	 * @param operationRemark
	 *            备注
	 */
	public Logable info(Object object, String operation, String operationRemark) {
		Assert.notNull(operation);
		return getLog(object, operation, operationRemark);
	}

	/**
	 * 修改状态
	 * 
	 * @param object
	 * @param operation
	 */
	public void status(Object object, Operation operation) {
		if (this.getLoggerParams().getStatusAnnotation() != null)
			setStatus(object, operation);
	}

	protected Logable getLog(Object object, String operation, String operationRemark) {
		return getLog(object, Operation.valueOf(operation), operationRemark);
	}

	protected Logable getLog(Object object, Operation operation) {
		Logable log = new OperationLog();
		log.setType(operation.toString());
		log.setTable(getLoggerParams().getTableName());
		log.setKey((Long) invokeMethod(object, getLoggerParams().getIdMethod()));
		log.setEnUU((Long) invokeMethod(object, getLoggerParams().getEnUUMethod()));
		log.setUserUU((Long) invokeMethod(object, getLoggerParams().getUserUUMethod()));
		return log;
	}

	protected Logable getLog(Object object, Operation operation, String operationRemark) {
		Logable log = getLog(object, operation);
		log.setDesc(operationRemark);
		return log;
	}

	protected Logable getLog(Object object, Long enUU, Long userUU, Operation operation) {
		Logable log = new OperationLog();
		log.setType(operation.toString());
		log.setTable(getLoggerParams().getTableName());
		log.setKey((Long) invokeMethod(object, getLoggerParams().getIdMethod()));
		log.setEnUU(enUU);
		log.setUserUU(userUU);
		return log;
	}

	protected Logable getLog(Long key, Long enUU, Long userUU, Operation operation) {
		Logable log = new OperationLog();
		log.setType(operation.toString());
		log.setTable(getLoggerParams().getTableName());
		log.setKey(key);
		log.setEnUU(enUU);
		log.setUserUU(userUU);
		return log;
	}

	protected Object invokeMethod(Object object, String param, Object... args) {
		try {
			Class<?>[] clss = new Class[] {};
			if (args != null && args.length > 0) {
				clss = new Class[args.length];
				for (int i = 0, j = args.length; i < j; i++) {
					clss[i] = args[i].getClass();
				}
			}
			Method method = object.getClass().getMethod(param, clss);
			return method.invoke(object, args);
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		return 0L;
	}

	public LoggerParams getLoggerParams() {
		return loggerParams;
	}

}
