/*CopyRright (c)2014: <www.usoftchina.com>
 */
package com.uas.platform.core.web.advice;

import java.sql.SQLRecoverableException;

import org.apache.log4j.Logger;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.ui.ModelMap;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;

import com.uas.platform.core.exception.IllegalOperatorException;
import com.uas.platform.core.exception.IllegalStatusException;
import com.uas.platform.core.exception.NotFoundException;
import com.uas.platform.core.exception.SystemError;
import com.uas.platform.core.exception.SystemException;

/**
 * <p>
 * 基于Application的异常处理,以AOP的形式注册到SpringMVC的处理链
 * </p>
 * <p>
 * 正常的业务流程,只需抛出对应的异常和相关的信息
 * </P>
 * <p>
 * 不同的错误，对应不同的方法来处理
 * </p>
 * <p>
 * 可以用不同的HttpStatus来表示具体的异常类型
 * </p>
 * <p>
 * 客户端可以基于对应的HttpStatus Code做出最有利于自己的处理
 * </p>
 * 
 * @author yingp
 * 
 */
@ControllerAdvice
public class ExceptionHandlerAdvice {

	private final static Logger logger = Logger.getLogger(ExceptionHandlerAdvice.class);

	/**
	 * 处理运行时抛出异常
	 * 
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(RuntimeException.class)
	public ResponseEntity<String> handleUnexpectedServerError(RuntimeException ex) {
		logger.error("RuntimeException", ex);
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/text; charset=utf-8");
		return new ResponseEntity<String>("\u7CFB\u7EDF\u9519\u8BEF", headers, HttpStatus.INTERNAL_SERVER_ERROR);
	}

	/**
	 * 处理权限判定失败情况
	 * 
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(AccessDeniedException.class)
	public ResponseEntity<String> handleAccessDeniedException(AccessDeniedException ex) {
		HttpHeaders headers = new HttpHeaders();
		logger.error("AccessDeniedException", ex);
		headers.add("Content-Type", "application/text; charset=utf-8");
		return new ResponseEntity<String>(ex.getMessage(), headers, HttpStatus.FORBIDDEN);
	}

	/**
	 * 处理连接池的连接失效抛出异常
	 * 
	 * @see SQLRecoverableException
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(SQLRecoverableException.class)
	public ResponseEntity<String> handleSQLRecoverableException(SQLRecoverableException ex) {
		logger.error("SQLRecoverableException", ex);
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/text; charset=utf-8");
		return new ResponseEntity<String>("\u7CFB\u7EDF\u9519\u8BEF", headers, HttpStatus.INTERNAL_SERVER_ERROR);
	}

	/**
	 * 处理参数错误抛出异常
	 * 
	 * @see IllegalArgumentException
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(IllegalArgumentException.class)
	public ResponseEntity<String> handleIllegalArgumentException(IllegalArgumentException ex) {
		logger.error("IllegalArgumentException", ex);
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/text; charset=utf-8");
		return new ResponseEntity<String>(StringUtils.isEmpty(ex.getMessage()) ? "\u53C2\u6570\u9519\u8BEF" : ex.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
	}

	/**
	 * 处理由于状态错误而抛出的异常
	 * 
	 * @see IllegalStatusException
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(IllegalStatusException.class)
	public ResponseEntity<String> handleIllegalStatusException(IllegalStatusException ex) {
		logger.error("IllegalStatusException", ex);
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/text; charset=utf-8");
		return new ResponseEntity<String>(ex.getMessage(), headers, HttpStatus.BAD_REQUEST);
	}

	/**
	 * 处理由于错误操作而抛出的异常
	 * 
	 * @see IllegalOperatorException
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(IllegalOperatorException.class)
	public ResponseEntity<String> handleIllegalOperatorException(IllegalOperatorException ex) {
		logger.error("IllegalOperatorException", ex);
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/text; charset=utf-8");
		return new ResponseEntity<String>(ex.getMessage(), headers, HttpStatus.BAD_REQUEST);
	}

	/**
	 * 处理由于没有找到实体而抛出的异常
	 * 
	 * @see NotFoundException
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(NotFoundException.class)
	public ResponseEntity<String> handleNotFoundException(NotFoundException ex) {
		logger.error("NotFoundException", ex);
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/text; charset=utf-8");
		return new ResponseEntity<String>(ex.getMessage(), headers, HttpStatus.NOT_FOUND);
	}

	/**
	 * 处理由于该用户而抛出的异常
	 * 
	 * @see UsernameNotFoundException
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(UsernameNotFoundException.class)
	public ResponseEntity<String> handleUserNotFoundException(UsernameNotFoundException ex) {
		logger.error("UsernameNotFoundException", ex);
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/text; charset=utf-8");
		return new ResponseEntity<String>(ex.getMessage(), headers, HttpStatus.NOT_FOUND);
	}

	/**
	 * 处理已捕获，需明确传达给客户端的异常
	 * 
	 * @see SystemException
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(SystemException.class)
	public ResponseEntity<String> handleSystemException(SystemException ex) {
		logger.error("SystemException", ex);
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/text; charset=utf-8");
		return new ResponseEntity<String>(ex.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
	}

	/**
	 * 处理已捕获异常，明确传达给客户端错误码、错误信息
	 * 
	 * @see SystemError
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(SystemError.class)
	public ResponseEntity<ModelMap> handleSystemError(SystemError ex) {
		logger.error("SystemError", ex);
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		ModelMap map = new ModelMap();
		map.put("errCode", ex.getError().code());
		map.put("errMsg", ex.getError().msg());
		return new ResponseEntity<ModelMap>(map, headers, HttpStatus.BAD_REQUEST);
	}

}
